package org.seatiger.clustering;

import java.awt.Color;
import java.awt.Graphics;
import java.util.Arrays;

import javax.swing.JFrame;
import javax.swing.JPanel;

import org.seatiger.pso.*;

public class TurnStar extends JPanel
{
   private static final int POINT_RADIUS = 3;
   private Range[] extents = {new Range(-1, 1), new Range(-1, 1)};
   private double[][] points;
   
   public TurnStar(int numPoints)
   {
      generatePoints(numPoints);
      findClusters();
   }
   

   
   private void findClusters()
   {
      boolean[] processed = new boolean[points.length];
      int n = 3;
      int blockSize = (int) Math.pow(2, n);
      
      for (int i = 0; i < 1/*points.length*/; i++)
      {
         if (processed[i])
            continue;
         
         Distance[] distances = new Distance[points.length];
         for (int j = 0; j < points.length; j++)
         {
            distances[j] = new Distance(j, VectorMath.length(
                  VectorMath.subtract(points[i], points[j])));
         }

         Arrays.sort(distances);
         
         for (int j = 0; j < points.length; j += blockSize)
         {
            double[] derivatives = new double[blockSize];
            int stopPoint = blockSize / 2;
            
            for (int k = 0; k < blockSize; k++)
               derivatives[k] = distances[j + k].distance;
            
            for (int pass = 0; pass < n; pass++)
            {
               for (int k = 0; k < stopPoint; k++)
                  derivatives[k] = derivatives[2 * k] - derivatives[2 * k + 1]; 
               
               stopPoint /= 2;
            }

            System.out.println(derivatives[0]);
         }
      }
   }
   
   
   
   
   public static void main(String[] args)
   {
      JFrame frame = new JFrame();
      frame.setSize(600, 600);
      frame.getContentPane().add(new TurnStar(1000));
      frame.setVisible(true);
      frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
   }
   
   public void paintComponent(Graphics g)
   {
      Range[] viewArea = {new Range(0, getWidth()), new Range(0, getHeight())};
      g.setColor(Color.BLUE);
      
      for (double[] point : points)
      {
         double[] screenPoint = SwarmDisplay2D.convert(extents, point, viewArea);
         g.fillOval((int) screenPoint[0] - POINT_RADIUS, 
               (int) screenPoint[1] - POINT_RADIUS, 
               POINT_RADIUS * 2 + 1, POINT_RADIUS * 2 + 1);
      }
   }
   
   private void generatePoints(int numPoints)
   {
      points = new double[numPoints][2];
      int pointsGenerated = 0;
      
      while (pointsGenerated < numPoints)
      {
         double x = Math.random() * 2 - 1;
         double y = Math.random() * 2 - 1;
         
         double height = Math.random() * 2 - 1;
         
         if (height > Math.sin(3 * Math.PI * x) && 
               height > Math.sin(3 * Math.PI * y))
         {
            points[pointsGenerated][0] = x;
            points[pointsGenerated][1] = y;
            pointsGenerated++;
         }
      }
   }
   
   private class Distance implements Comparable<Distance>
   {
      public int toPoint;
      public double distance;
      
      public Distance(int toPoint, double distance)
      {
         this.toPoint = toPoint;
         this.distance = distance;
      }
      
      public int compareTo(Distance o)
      {
         if (o.distance > distance)
            return -1;
         else if (o.distance < distance)
            return 1;
         else 
            return 0;
      }
   }
}
