package org.seatiger.pso;

/**
 * DeJong's test function 2.  This has a large, flat ridge. 
 * Some GAs have difficulty moving along the ridge to find 
 * the optimum point.
 * 
 * The equation for this function is: 100((x1)^2 - (x2))^2 + (1 - (x1)^2)
 *   
 * @author Stefan Bird <stbird@seatiger.org>
 */
public class DeJongF2 implements FitnessFunction
{

   public int getNumVariables()
   {
      return 2;
   }

   public double getFitness(double[] location)
   {
      double partA = location[0] * location[0] - location[1];
      double partB = 1 - location[0];
      
      return -100 * partA * partA + partB * partB; 
   }

   /**
    * Returns the preferred range of this fitness function, which is
    * [-2.048, 2.048] for both variables
    * 
    * @see FitnessFunction#getPreferredRanges()
    */
   public Range[] getPreferredRanges()
   {
      Range range = new Range(-2.048, 2.048);
      
      return new Range[] {range, range};
   }

}
