package org.seatiger.pso;

import java.util.Arrays;

/**
 * De Jong's test function 3. A series of steps to the global 
 * optimum. If a GA doesn't find the next step, it can become 
 * stuck on a sub-optimal plateau.
 * 
 * @author Stefan Bird <stbird@seatiger.org>
 */
public class DeJongF3 implements FitnessFunction
{
   /** The number of parameters (dimensions) in this problem */
   private int parameters;
   
   /** The preferred range of this fitness function */
   private final static Range range = new Range(-5.12, 5.12); 
   
   /**
    * Creates a DF3 fitness function 
    *
    * @param parameters How many parameters (dimensions) the problem
    * should be in 
    */
   public DeJongF3(int parameters)
   {
      this.parameters = parameters;
   }
   
   /**
    * Returns the number of parameters expected by this fitness
    * function. 
    * 
    * @return The number of parameters specified in the constructor
    * @see org.seatiger.pso.FitnessFunction#getNumVariables()
    */
   public int getNumVariables()
   {
      return parameters;
   }

   /**
    * Returns the fitness of the given location. 
    * 
    * @param location The location to test
    * @return The fitness value of that location
    * @see org.seatiger.pso.FitnessFunction#getFitness(double[])
    */
   public double getFitness(double[] location)
   {
      int sum = 0;
      
      for (double x : location)
         sum += (int) x;
      
      return sum;
   }

   /**
    * Returns the preferred range of this fitness function, 
    * which is [-5.12, 5.12] for all parameters
    * 
    * @return The preferred range
    * @see org.seatiger.pso.FitnessFunction#getPreferredRanges()
    */
   public Range[] getPreferredRanges()
   {
      Range[] result = new Range[parameters];
      Arrays.fill(result, range);
      
      return result;
   }

}
