package org.seatiger.pso;

/**
 * DeJong's test function 5 - many local optima (fox-holes) surrounded 
 * by flat areas
 * 
 * @author Stefan Bird <stbird@seatiger.org>
 */
public class DeJongF5 implements FitnessFunction
{

   public int getNumVariables()
   {
      return 2;
   }

   public double getFitness(double[] location)
   {
      int[][] a = new int[2][25];
      int next1 = -32;
      int next2 = -32;
      
      for (int j = 0; j < 25; j++)
      {
         a[0][j] = next1;
         a[1][j] = next2;
         
         next1 += 16;
         if (next1 > 32)
         {
            next1 = -32;
            next2 += 16;
         }
      }
         
      double sum = 0.002;
      
      for (int j = 0; j < 25; j++)
      {
         double temp1 = j + 1;
         
         for (int i = 0; i < 2; i++)
         {
            double temp2 = location[i] - a[i][j];
            temp1 += temp2 * temp2 * temp2 * temp2 * temp2 * temp2;
         }
         
         sum += 1 / temp1;
      }
      
      return sum;
   }

   /**
    * Returns the preferred range of this fitness function, which is
    * [-65.536, 65.536] for both variables
    * 
    * @see FitnessFunction#getPreferredRanges()
    */
   public Range[] getPreferredRanges()
   {
      Range range = new Range(-65.536, 65.536);
      
      return new Range[] {range, range};
   }

}
