package org.seatiger.pso;

import java.util.Arrays;

/**
 * The default knapsack representation. This implements a 0/1 knapsack, with a 
 * decision variable for each item type. An item is considered to be "in" the
 * knapsack if the integer value of any of the decision variables is equal to 
 * that item's index.  
 * 
 * @author Stefan Bird <stbird@seatiger.org>
 */
public class DefaultKnapsackRepresentation implements KnapsackRepresentation
{
   public int[] decode(double[] location, KnapsackProblem problem)
   {
      int[] result = new int[problem.getNumItemTypes()];
      
      for (double d : location)
      {
         int value = (int) d;
         
         if (value >= 0 && value < result.length)
            result[value] = 1;
      }
      
      return result;
   }

   public int getNumVariables(KnapsackProblem problem)
   {
      return problem.getMaxItems();
   }

   public Range[] getPreferredRanges(KnapsackProblem problem)
   {
      Range[] result = new Range[problem.getMaxItems()];
      Arrays.fill(result, new Range(0, problem.getNumItemTypes()));
      
      return result;
   }
}
