package org.seatiger.pso;

/**
 * Maintains a count of the number of fitness evaluations
 * 
 * @author Stefan Bird <stbird@seatiger.org>
 */
public class EvaluationCounter implements FitnessFunction
{
   /** The fitness function that actually does the evaluation */
   private FitnessFunction delegate;

   /** The number of evaluations */
   private int evaluations = 0;
   
   /**
    * Creates a new EvaluationCounter
    * 
    * @param delegate The delelegate fitness function
    */
   public EvaluationCounter(FitnessFunction delegate)
   {
      if (delegate == null)
         throw new NullPointerException("Delegate cannot be null");
      
      this.delegate = delegate;
   }
   
   /** 
    * Returns the number of evaluations since the counter was last
    * reset
    * 
    * @return The number of evaluations
    */
   public int getEvaluations()
   {
      return evaluations;
   }
   
   /**
    * Sets the evaluation counter to 0
    */
   public void resetEvaluations()
   {
      evaluations = 0;
   }
   
   public int getNumVariables()
   {
      return delegate.getNumVariables();
   }

   public double getFitness(double[] location)
   {
      evaluations++;
      return delegate.getFitness(location);
   }

   public Range[] getPreferredRanges()
   {
      return delegate.getPreferredRanges();
   }
}
