package org.seatiger.pso;

/**
 * The Himmelblau fitness function. This has 4 equal global optima
 * 
 * @author Stefan Bird <stbird@seatiger.org>
 */
public class Himmelblau implements FitnessFunction
{
   /** 
    * Returns the number of variables (2)
    * 
    * @see FitnessFunction#getNumVariables()
    */
   public int getNumVariables()
   {
      return 2;
   }

   /**
    * @see FitnessFunction#getFitness(double[])
    */
   public double getFitness(double[] location)
   {
      double x = location[0];
      double y = location[1];
      
      double partA = x * x + y - 11;
      double partB = x + y * y - 7;
      
      return 200 - partA * partA - partB * partB;
   }

   /**
    * Returns the preferred range of this fitness function, which is
    * [-6, 6] for both variables
    * 
    * @see FitnessFunction#getPreferredRanges()
    */
   public Range[] getPreferredRanges()
   {
      Range range = new Range(-6, 6);
      
      return new Range[] {range, range};
   }

}
