package org.seatiger.pso;

/**
 * A representation of the knapsack problem
 * 
 * @author Stefan Bird <stbird@seatiger.org>
 */
public interface KnapsackRepresentation
{
   /**
    * Returns the number of variables in the decision space 
    * 
    * @param problem The given knapsack problem
    * @return The number of input variables required by this function
    */
   public int getNumVariables(KnapsackProblem problem);
   
   /**
    * Decodes the given point. This function may be asked
    * to evaluate points outside the ranges given by 
    * {@link #getPreferredRanges(KnapsackProblem)}.
    * This function should return null in these situations.
    * 
    * @param location The location of the point to decode. This will have
    * the same number of elements as {@link #getNumVariables(KnapsackProblem)}
    * @param problem The given knapsack problem
    * @return The quantity for each item - eg [0, 2, 4] means 2 of item 1 and 4 of
    * item 2.
    * @see FitnessFunction#getNumVariables()
    */
   public int[] decode(double[] location, KnapsackProblem problem);

   /**
    * Returns the preferred range of each dimension for this encoding.
    * The preferred range is the range of values that the search should be
    * constrained by. The optimum solution should normally exist within the
    * preferred range, although this is not a requirement.
    *  
    * <p>Note: This function returns the <i>preferred</i> range only. The decoder
    * may be asked to decode a point outside this range, in which case it may return
    * null. 
    * @param problem The given knapsack problem
    * @return An array of {@link Range}s, one for each dimension of the encoding
    */
   public Range[] getPreferredRanges(KnapsackProblem problem);
}
