package org.seatiger.pso;

import java.awt.*;
import java.util.ArrayList;
import java.util.LinkedHashSet;

import uchicago.src.sim.gui.*;

/**
 * Displays arbitary labels
 *  
 * @author Stefan Bird <stbird@seatiger.org>
 */
public class LabelDisplay implements Displayable
{
   /** The labels to display */
   private LinkedHashSet<Label> labels = new LinkedHashSet<Label>();
   
   /** The swarm that the display is labelling */
   private Swarm swarm;
   
   /** 
    * Creates a new LabelDisplay for the given Swarm
    * 
    * @param swarm The swarm to display labels for
    */
   public LabelDisplay(Swarm swarm)
   {
      this.swarm = swarm;
   }
   
   public void drawDisplay(SimGraphics g)
   {
      Range[] variableRange = swarm.getVariableRange();
      Range[] displayRange = {new Range(0, g.getDisplayWidth()), 
            new Range(0, g.getDisplayHeight())};

      Graphics2D g2 = g.getGraphics();
      
      
      for (Label label : labels)
      {
         double[] displayPoint = SwarmDisplay2D.convert(variableRange, 
               label.position, displayRange);
         
         g2.setColor(label.color);
         g2.drawString(label.text, (int) displayPoint[0], (int) displayPoint[1]);
      }
   }

   /**
    * Adds a label
    * 
    * @param position The point being labeled, in model coordiantes
    * @param text The text of the label
    * @param color The colour of the label
    */
   public void addLabel(double[] position, String text, Color color)
   {
      labels.add(new Label(position, text, color));
   }
   
   /**
    * Removes all labels
    */
   public void clearLabels()
   {
      labels.clear();
   }


   /** Holds the information for a single label */
   private class Label
   {
      /** The text of the label */
      public String text;
      /** The position of the label in model coordinates */
      public double[] position;
      /** The colour of the label */
      public Color color;
      
      /**
       * Creates a new label
       * 
       * @param position The location of the label in model coordinates
       * @param text The text of the label
       * @param color The colour of the label
       */
      public Label(double[] position, String text, Color color)
      {
         this.text = text;
         this.color = color;
         this.position = new double[position.length];
         System.arraycopy(position, 0, this.position, 0, position.length);
      }
   }
   
   public ArrayList getDisplayableInfo() { return new ArrayList(); }
   public void viewEventPerformed(ViewEvent evt) { }
   public Dimension getSize() { return new Dimension(200, 200); }
}
