package org.seatiger.pso;

/**
 * Represents a leaf in the tree
 * 
 * @author Stefan Bird <stbird@seatiger.org>
 */
public class LeafNode extends TreeNode
{
   /** The location that this node holds. Only leaf nodes actually
    * hold a point */
   private double[] point = null;
   
   /**
    * Creates a new TreeNode representing the given point
    *
    * @param point The point to represent
    */
   public LeafNode(double[] point)
   {
      super(point.length);
      
      this.point = new double[point.length];
      System.arraycopy(point, 0, this.point, 0, point.length);
   }
 
   /**
    * The data point held by this node. 
    * 
    * @return The point held by this node
    */
   public double[] getPoint()
   {
      double[] result = new double[point.length];
      System.arraycopy(point, 0, result, 0, point.length);
      
      return result;
   }
   
   @Override
   public double[] getClosestPointTo(double[] target)
   {
      return getPoint();
   }

   /**
    * Returns true if <code>o</code> is a LeafNode representing
    * the same point
    */
   @Override
   public boolean equals(Object o)
   {
      if (o.getClass() != getClass())
         return false;
      
      LeafNode other = (LeafNode) o;
      
      if (other.getNumDimensions() != getNumDimensions())
         return false;
      
      double[] otherPoint = other.getPoint();
      for (int i = 0; i < otherPoint.length; i++)
      {
         if (otherPoint[i] != point[i])
            return false;
      }
      
      return true;
   }

   @Override
   public int hashCode()
   {
      long result = 0;
      for (int i = 0; i < point.length; i++) 
      {
         result ^= Double.doubleToLongBits(point[i]);
         // Without the bitshift, LeafNodes at points like (0.5, 0.5) will have
         // hashcodes of 0 (because of the XOR)
         result >>= 1;
      }

      return (int) result;
   }
}
