/**
 * 
 */
package org.seatiger.pso;

import java.util.Arrays;

/**
 * Performs a linear regression
 * 
 * @author Stefan Bird <stbird@seatiger.org>
 */
public class LinearRegression implements FitnessFunction
{
   /** The range of each variable */
   private final static Range range = new Range(0, 250); 
   
   /** The actual A of Ax + B */
   private double actualA;
   /** The actual B of Ax + B */
   private double actualB;
   
   /**
    * Initialises the goal values
    * 
    * @param actualA The actual A of Ax + B
    * @param actualB The actual B of Ax + B
    */
   public LinearRegression(double actualA, double actualB)
   {
      this.actualA = actualA;
      this.actualB = actualB;
   }

   /* (non-Javadoc)
    * @see org.seatiger.pso.FitnessFunction#getNumVariables()
    */
   public int getNumVariables()
   {
      return 2;
   }

   public double getFitness(double[] location)
   {
      double error = 0;

      
      for (int x = 0; x < 20; x++)
      {
         double testValue = location[0] * x + location[1];
         double actualValue = actualA * x + actualB;
         
         double diff = testValue - actualValue;
         
         error += diff * diff;        
      }
      
      return -error;
   }

   /**
    * Returns the preferred range of this fitness function, 
    * which is [0, 250] for all parameters
    * 
    * @return The preferred range
    * @see org.seatiger.pso.FitnessFunction#getPreferredRanges()
    */
   public Range[] getPreferredRanges()
   {
      Range[] result = new Range[2];
      Arrays.fill(result, range);
      
      return result;
   }

}
