package org.seatiger.pso;

/**
 * Represents an integer. Java's Integer class is not mutable, 
 * so a new instance must be created every time the value is 
 * changed. 
 * 
 * @author Stefan Bird <stbird@seatiger.org>
 */
public class MutableInteger
{
   /** The number being represented */
   int number;
   
   /**
    * Creates a new MutableInteger
    *
    * @param number The number to represent
    */
   public MutableInteger(int number)
   {
      this.number = number;
   }
   
   /**
    * Returns the number being represented 
    * 
    * @return The number being represented
    */
   public int getValue()
   {
      return number;
   }
   
   /**
    * Sets the value held by this instance
    * 
    * @param number The value to set
    */
   public void setValue(int number)
   {
      this.number = number;
   }
   
   /**
    * Increments the value by 1
    * 
    * @return The new value after incrementing
    */
   public int increment()
   {
      return this.increment(1);
   }

   /**
    * Increments the value by <code>amount</code>
    * 
    * @param amount The amount to increment by 
    * @return The new value after incrementing
    */
   public int increment(int amount)
   {
      number += amount;
      
      return number; 
   }
   
   /**
    * Decrements the value by 1
    * 
    * @return The new value after decrementing
    */
   public int decrement()
   {
      return this.decrement(1);
   }

   /**
    * Decrements the value by <code>amount</code>
    * 
    * @param amount The amount to decrement by 
    * @return The new value after decrementing
    */
   public int decrement(int amount)
   {
      number -= amount;
      
      return number; 
   }
}