package org.seatiger.pso;

import java.util.Set;

/**
 * Manages a neighbourhood of particles
 * 
 * @author Stefan Bird <stbird@seatiger.org>
 */
public interface Neighborhood
{
   /** 
    * Returns the particles that are in the same neighbourhood as 
    * <code>p</code>. If <code>p</code> has not been registered with the 
    * neighbourhood, this function should return null
    * 
    * @param p The particle to get the neighbourhood of
    * @return The set of particles in the same neighbourhood
    */
   public Set<Particle> getNeighborhoodOf(Particle p);
   
   /**
    * Registers <code>p</code> with the neighbourhood
    * 
    * @param p The Particle to register
    */
   public void register(Particle p);
   
   /**
    * Deregisters <code>p</code> from the neighbourhood
    * 
    * @param p The Particle to deregister
    */
   public void deregister(Particle p);
   
   /**
    * Deregisters all particles from the neighbourhood
    */
   public void deregisterAll();
   
}
