package org.seatiger.pso;

import java.util.Collection;

/**
 * Returns the particle's neighborhood best point as the attractor
 *  
 * @author Stefan Bird <stbird@seatiger.org>
 */
public class NeighborhoodBestAttractor implements ParticleAttractor
{
   /** The neighbourhood model */
   private Neighborhood neighborhood;
   
   /** The results for ANPSO/ESPSO were generated with a slight error in 
    * the original code - the attractor point was the *current* location
    * of the neighbourhood best particle, not the best location as it should 
    * have been. By setting this true, the original (incorrect) behaviour
    * will be used, allowing results comparable to the ones presented in the 
    * paper to be presented  
    */
   public static boolean COMPATIBILITY_MODE = false;
   
   /**
    * Creates a new NeighborhoodBestAttractor
    *
    * @param neighborhood The neighbourhood model to use
    */
   public NeighborhoodBestAttractor(Neighborhood neighborhood)
   {
      this.neighborhood = neighborhood;
   }
   
   public double[] getAttractorPoint(Particle p, Collection<Particle> allParticles)
   {
      Particle bestParticle = null;
      
      for (Particle particle : neighborhood.getNeighborhoodOf(p))
      {
         if (bestParticle == null ||
               particle.getPersonalBestFitness()[0] > 
               bestParticle.getPersonalBestFitness()[0])
            bestParticle = particle;
      }

      if (COMPATIBILITY_MODE)
         return bestParticle.getLocation();
      else
         return bestParticle.getPersonalBestLocation();
   }

}
