package org.seatiger.pso;

import java.awt.Color;
import java.awt.Dimension;
import java.util.ArrayList;
import java.util.Set;

import uchicago.src.sim.gui.*;

/**
 * Displays the neighbourhood relationships between particles. If multiple
 * neighborhoods are being used, create an instance of this class for each
 * neighborhood
 * 
 * @author Stefan Bird <stbird@seatiger.org>
 */
public class NeighborhoodDisplay2D implements Displayable
{
   /** The neighborhood of the particles */
   private Neighborhood neighborhood;
   /** The Swarm the particles belong to */
   private Swarm swarm;
   /** The colour to connect the particles with */
   private Color lineColor = new Color(0, 0, 64);
   
   /**
    * Creates a new NeighborhoodDisplay2D
    *
    * @param swarm The swarm the particles belong to
    * @param neighborhood The neighborhood the particles belong to
    */
   public NeighborhoodDisplay2D(Swarm swarm, Neighborhood neighborhood)
   {
      this.swarm = swarm;
      this.neighborhood = neighborhood;
   }
   
   /**
    * Draws a dashed line between each particle and its neighbours
    * 
    * @see uchicago.src.sim.gui.Displayable#drawDisplay(uchicago.src.sim.gui.SimGraphics)
    */
   public void drawDisplay(SimGraphics g)
   {
      Range[] particleRange = swarm.getVariableRange();
      Range[] displayRange = {new Range(0, g.getDisplayWidth()), 
            new Range(0, g.getDisplayHeight())};

      for (Particle p : swarm.getParticles())
      {
         double[] temp = SwarmDisplay2D.convert(particleRange, p.getLocation(),
               displayRange);
         int[] pCoords = {(int) temp[0], (int) temp[1]};
         
         Set<Particle> neighbors = neighborhood.getNeighborhoodOf(p);
         if (neighbors == null)
            continue;
         
         for (Particle other : neighbors)
         {
            temp = SwarmDisplay2D.convert(particleRange, other.getLocation(), 
                  displayRange);
            int[] otherCoords = {(int) temp[0], (int) temp[1]};
            
            g.drawLink(lineColor, pCoords[0], otherCoords[0], 
                  pCoords[1], otherCoords[1]);
         }
      }
   }

   /**
    * @see uchicago.src.sim.gui.Displayable#getSize()
    */
   public Dimension getSize()
   {
      return new Dimension(400, 400);
   }
   
   /**
    * @see uchicago.src.sim.gui.Displayable#getDisplayableInfo()
    */
   public ArrayList getDisplayableInfo()
   {
      return new ArrayList();
   }

   /**
    * @see uchicago.src.sim.gui.Displayable#viewEventPerformed(uchicago.src.sim.gui.ViewEvent)
    */
   public void viewEventPerformed(ViewEvent evt) { }
}
