package org.seatiger.pso;


/**
 * A regression for polynomial functions
 * 
 * @author Stefan Bird <stbird@seatiger.org>
 */
public class PolynomialRegression extends RegressionAdapter 
{
   /** The order of the polynomial */
   private int order;
   
   /**
    * Creates a new PolynomialRegression of the given order
    * 
    * @param order The order of the polynomial, eg ax^2 + bx + c
    * has an order of 2
    * @throws IllegalArgumentException if <code>order</code> < 0
    */
   public PolynomialRegression(int order)
   {
      if (order < 0)
         throw new IllegalArgumentException("order cannot be < 0");
      
      this.order = order;
      setPreferredRanges(null);
   }

   public int getNumConstants()
   {
      return order + 1;
   }

   public int getNumVariables()
   {
      return 1;
   }

   /**
    * Sets the constants. The constants array is read from left to 
    * right, meaning that for f(x) = ax^2 + bx + c, element 0 would
    * be a, element 1 would be b and element 2 would be c
    */
   @Override
   public void setConstants(double[] constants)
   {
      super.setConstants(constants);
   }

   public double calculate(double[] location)
   {
      double result = 0;
      double x = location[0];
      double[] constants = getConstants();
      
      for (int power = order; power >= 0; power--)
         result += constants[order - power] * Math.pow(x, power);
      
      return result;
   }
   
   @Override
   public void setPreferredRanges(Range[] preferredRanges)
   {
      super.setPreferredRanges(preferredRanges);
   }
}
