package org.seatiger.pso;

import java.util.Arrays;

/**
 * A difficult function with many local optima
 * 
 * @author Stefan Bird <stbird@seatiger.org>
 */
public class Rastrigin implements FitnessFunction
{
   /** The number of parameters (dimensions) in this problem */
   private int parameters;
   /** The preferred range of this fitness function */
   private final static Range range = new Range(-5.12, 5.12); 
   
   /**
    * Creates a Rastrigin fitness function 
    *
    * @param parameters How many parameters (dimensions) the problem
    * should be in 
    */
   public Rastrigin(int parameters)
   {
      this.parameters = parameters;
   }
   
   /**
    * @see org.seatiger.pso.FitnessFunction#getNumVariables()
    */
   public int getNumVariables()
   {
      return parameters;
   }

   /**
    * @see org.seatiger.pso.FitnessFunction#getFitness(double[])
    */
   public double getFitness(double[] location)
   {
      double sum = 0;
      
      for (int i = 0; i < location.length; i++)
      {
         sum += location[i] * location[i] - 10 * Math.cos(2 * Math.PI * 
               location[i]) + 10;
      }
      
      return -sum;
   }

   /**
    * Returns the preferred range of this fitness function, 
    * which is [-5.12, 5.12] for all parameters
    * 
    * @return The preferred range
    * @see org.seatiger.pso.FitnessFunction#getPreferredRanges()
    */
   public Range[] getPreferredRanges()
   {
      Range[] result = new Range[parameters];
      Arrays.fill(result, range);
      
      return result;
   }

}
