package org.seatiger.pso;

/**
 * A regression function, to be used with {@link RegressionEvaluator}
 * 
 * @author Stefan Bird <stbird@seatiger.org>
 */

public interface Regression
{
   /** 
    * Returns the number of constants in this regression.
    * 
    * In a linear regression (f(x) = ax + b) there are two
    * constants: a and b, so this function would return 2
    * 
    * @return The number of constants this regression uses
    */
   public int getNumConstants();
   
   /**
    * Returns the number of variables in this regression
    * 
    * In a linear regression (f(x) = ax + b) there is one
    * variable: x, so this function would return 1
    * 
    * @return The number of variables this regression uses
    */
   public int getNumVariables();
   
   /**
    * Sets the constants of the regression
    * 
    * In a linear regression (f(x) = ax + b), <code>constants[0]</code>
    * would represent <code>a</code> and <code>constants[1]</code> would
    * represent <code>b</code>
    * 
    * @param constants The values for the constants
    */
   public void setConstants(double[] constants);
   
   /**
    * Calculates the value of location
    * 
    * @param location The location to calculate the value of
    * @return The calculated value
    */
   public double calculate(double[] location);
   
   /**
    * Returns the preferred range of the constants
    * 
    * @return The preferred range of the constants
    * @see FitnessFunction#getPreferredRanges()
    */
   public Range[] getPreferredRanges();
}
