package org.seatiger.pso;

/**
 * The source of the data we're trying to fit a regression to
 * 
 * @author Stefan Bird <stbird@seatiger.org>
 */
public interface RegressionSource
{
   /**
    * Returns the value at the specified location
    * 
    * @param location One of the locations returned by 
    * {@link #getTestPoints()}
    * @return The value at the specified location
    * @throws IllegalArgumentException if location is not one of the
    * locations returned by {@link #getTestPoints()}. Throwing this 
    * exception is optional; certain RegressionSources may choose to 
    * interpolate (or some other method) to calculate values outside of 
    * the test points. 
    */
   public double calculate(double[] location);
   
   /**
    * Returns the array of points that the function can test. For
    * example if we can calculate the value at points (5, 3) and (4, 6)
    * this function should return:
    * 
    * { {5, 3}, {4, 6} }
    * 
    * @return The array of test locations
    */
   public double[][] getTestPoints();
}
