package org.seatiger.pso;

import java.awt.*;
import java.awt.geom.Area;
import java.awt.geom.Ellipse2D;
import java.util.*;
import java.util.List;

import uchicago.src.sim.gui.*;

/**
 * Displays a number on each species seed saying how long it's been
 * there
 * 
 * @author Stefan Bird <stbird@seatiger.org>
 */
public class SpeciesAgeDisplay implements Displayable
{
   private Range[] variableRange;
   
   private SPSO classifier;
   
   private HashMap<double[], Integer> speciesAges = new HashMap<double[], Integer>();
   
   /**
    */
   public SpeciesAgeDisplay(Swarm swarm, SpeciesNeighborhood neighborhood,
         SPSO classifier)
   {
      this.classifier = classifier;
      neighborhood.addObserver(new Observer() {
         public void update(Observable source, Object arg)
         {
            updateAges();
         }});
   
      variableRange = swarm.getVariableRange();
   }

   public void drawDisplay(SimGraphics g)
   {
      Graphics2D g2 = g.getGraphics();
      
      Range[] displayRange = { new Range(0, g.getDisplayWidth()), 
            new Range(0, g.getDisplayHeight())};
      
      g2.setColor(Color.RED);
      double radius = classifier.getRadius();
      double[] ovalSize = SwarmDisplay2D.convert(variableRange, 
            new double[] {radius, radius}, displayRange);
      ovalSize = VectorMath.subtract(ovalSize, SwarmDisplay2D.convert(
            variableRange, new double[2], displayRange));
      
      int ovalWidth = (int) ovalSize[0];
      int ovalHeight = (int) ovalSize[1];
      
      Shape originalClip = g2.getClip();
      
      for (double[] position : speciesAges.keySet())
      {
         Integer age = speciesAges.get(position);
         
         double[] displayPoint = SwarmDisplay2D.convert(variableRange, position,
               displayRange);
         int x = (int) displayPoint[0];
         int y = (int) displayPoint[1];

         g2.drawString(age.toString(), x, y);
      }

      g2.setClip(0, 0, g.getDisplayWidth(), g.getDisplayHeight());
      
      for (Particle p : classifier.getSpeciesSeeds())
      {
         double[] displayPoint = SwarmDisplay2D.convert(variableRange, 
               p.getPersonalBestLocation(), displayRange);
         int x = (int) displayPoint[0];
         int y = (int) displayPoint[1];
         
         g2.drawOval(x - ovalWidth, y - ovalHeight, 
               ovalWidth * 2, ovalHeight * 2);
      }
      
      g2.setClip(originalClip);
   }

   /**
    * Updates the age of each species
    */
   private void updateAges()
   {
      List<Particle> seeds = classifier.getSpeciesSeeds();
      
      if (seeds == null || seeds.size() == 0)
      {
         speciesAges.clear();
         return;
      }

      ArrayList<double[]> currPositions = new ArrayList<double[]>();
      
      for (Particle p : seeds)
      {
         double[] position;
         
         if (classifier.isUsePBest())
            position = p.getPersonalBestLocation();
         else
            position = p.getLocation();
         
         currPositions.add(position);
         
         if (!speciesAges.containsKey(position))
            speciesAges.put(position, 1);
         else
            speciesAges.put(position, speciesAges.get(position) + 1);
      }
      
      speciesAges.keySet().retainAll(currPositions);
   }
   
   public ArrayList getDisplayableInfo()
   {
      return new ArrayList();
   }

   public Dimension getSize()
   {
      return new Dimension(400, 400);
   }

   public void viewEventPerformed(ViewEvent evt) { }
}
