package org.seatiger.pso;

import java.util.Collection;
import java.util.Map;

/**
 * Classifies a number of particles into species
 * 
 * @author Stefan Bird <stbird@seatiger.org>
 */
public interface SpeciesClassifier
{
   /**
    * Classify the particles into species. 
    * 
    * @param particles The particles to be classified
    * @return A Map containing all of the particles in species as keys, with
    * an Integer saying which species number that particle belongs to. 
    * Particles with the same species number are considered to be in the same
    * species. The only restriction on the numbers is that they are unique
    * between species; the numbers themselves do not have to be sequential
    * or in any particularr order. Particles that don't belong to any species
    * should either not exist in the returned Map or be mapped to null.  
    */
   public Map<Particle, Integer> classify(Collection<Particle> particles);
}
