package org.seatiger.pso;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.util.ArrayList;
import java.util.List;

import uchicago.src.sim.gui.Displayable;
import uchicago.src.sim.gui.SimGraphics;
import uchicago.src.sim.gui.ViewEvent;

/**
 * Displays info about the species locations and ages
 * 
 * @author Stefan Bird <scbird@seatiger.org>
 */
public class SpeciesDisplay2D implements Displayable
{
   /** Used to convert from model to view coordinates */
   private SwarmDisplay2D swarmDisplay;
   /** The tracker we're monitoring */
   private SpeciesTracker classifier;
   
   /**
    * Creates a new ESPSO2Display
    * 
    * @param classifier The classifier to monitor
    * @param swarmDisplay The swarm display we're sitting on top of
    */
   public SpeciesDisplay2D(SpeciesTracker classifier, SwarmDisplay2D swarmDisplay)
   {
      if (swarmDisplay == null)
         throw new NullPointerException("Display cannot be null");
      this.swarmDisplay = swarmDisplay;
      if (classifier == null)
         throw new NullPointerException("Classifier cannot be null");
      this.classifier = classifier;
   }
   
   public void drawDisplay(SimGraphics sg)
   {
      Graphics g = sg.getGraphics();
      g.setColor(Color.RED);
      ESPSO2 espso2 = null;
      
      if (classifier.getDelegate() instanceof ESPSO2)
         espso2 = (ESPSO2) classifier.getDelegate();
      
      List<Species> species = classifier.getAllSpecies();
      for (Species currSpecies : species)
      {         
         int age = currSpecies.getBorn();
         double[] seed = currSpecies.getSeed();
         Point centre = swarmDisplay.convertToScreen(seed[0], seed[1]);
         
         String string = age + ", " + currSpecies.getLastMovement() + " (" + 
               Integer.toString(currSpecies.getParticles().size()) + ")";
         
         g.drawString(string, centre.x, centre.y);
      }
   }

   
   @SuppressWarnings("unchecked")
   public ArrayList getDisplayableInfo()
   {
      return new ArrayList();
   }

   public Dimension getSize()
   {
      return swarmDisplay.getSize();
   }

   public void viewEventPerformed(ViewEvent arg0)  { }
}
