package org.seatiger.pso;


/**
 * Represents a node within an <i>n</i> dimensional tree. Although a tree 
 * consists of branch nodes and leaf nodes, only the leaf node actually
 * contain data. 
 * 
 * @author Stefan Bird <stbird@seatiger.org>
 */
public abstract class TreeNode
{
   /** The parent of this node */
   private BranchNode parent;
   
   /** The number of dimensions of the points in the tree */
   private int dimensions;

   /**
    * Creates a new TreeNode with the specified number of dimensions
    *
    * @param dimensions The number of dimensions in the tree
    */
   protected TreeNode(int dimensions)
   {
      this.dimensions = dimensions;
   }
   
   /**
    * Returns the number of dimensions in this tree. This is the same
    * as the length of the point the node was created with
    * 
    * @return The number of dimensions in the tree
    */
   public int getNumDimensions()
   {
      return dimensions;
   }
   
   /**
    * Returns the parent of this treenode
    * 
    * @return The parent of this treenode
    */
   public BranchNode getParent()
   {
      return parent;
   }
   
   /**
    * Sets the parent of this tree node 
    * 
    * @param parent The new parent
    */
   protected void setParent(BranchNode parent)
   {
      this.parent = parent;
   }
   
   /**
    * Returns the closest point within the extents of this TreeNode
    * to <code>location</code>
    * 
    * @param target The location to find the closest point to
    * @return The closest point to <code>location</code>
    */
   public abstract double[] getClosestPointTo(double[] target);
}
