package org.seatiger.pso;

import java.awt.Dimension;
import java.awt.Image;
import java.awt.geom.Point2D;

/**
 * A fitness function that can directly return an image visualising
 * itself. The visualised space has (0, 0) as its upper left 
 * coordinate and <code>size</code> as its lower right
 * 
 * @author Stefan Bird <stbird@seatiger.org>
 */
public interface VisualizedFitnessFunction extends FitnessFunction
{
   /** 
    * Returns a visualisation for the fitness function 
    * 
    * @param size The size of the visualisation
    * @return An image visualising the function
    */
   public Image getVisualization(Dimension size);
   
   /**
    * Converts the given point into the visualisation space
    *  
    * @param point The point in the decision space to convert
    * @param size The size of the visualisation space 
    * @return The point in the 2D visualisation 
    */
   public Point2D convertToVisualSpace(double[] point, Dimension size);
}
