package org.seatiger.pso.test;

import java.io.IOException;
import java.io.StringReader;
import java.util.Arrays;

import junit.framework.TestCase;

import org.seatiger.pso.CSVSource;

/**
 * Tests the CSVSource class
 * 
 * @author Stefan Bird <stbird@seatiger.org>
 */
public class CSVSourceTest extends TestCase
{
   /** The test points the CSVSource is expected to return */
   private static final double[][] EXPECTED_TEST_POINTS = 
      { { 1, 1}, {2, 1}, {1, 2}, {2, 2} };  
   
   /** The expected result of {@link CSVSource#calculate(double[])} for
    * each of {@link #EXPECTED_TEST_POINTS} */
   private static final double[] EXPECTED_RESULTS = 
      { 2, 3, 3, 4 };

   /**
    * Creates a new CSVSourceTest
    */
   public CSVSourceTest()
   {
      super("CSVSource");
   }
   
   /**
    * Test method for 'org.seatiger.pso.CSVSource.getTestPoints()'
    * 
    * @throws IOException If creating the CSVSource fails
    */
   public void testGetTestPoints() throws IOException
   {
      CSVSource source = createCSVSource();
      
      double[][] testPoints = source.getTestPoints();
      
      assertEquals(EXPECTED_TEST_POINTS.length, testPoints.length);
      
      for (int i = 0; i < EXPECTED_TEST_POINTS.length; i++)
         assertTrue(Arrays.equals(EXPECTED_TEST_POINTS[i], testPoints[i]));
   }   
   
   /**
    * Test method for 'org.seatiger.pso.CSVSource.calculate(double[])'
    * 
    * @throws IOException If creating the CSVSource fails
    */
   public void testCalculate() throws IOException
   {
      CSVSource source = createCSVSource();
      
      for (int i = 0; i < EXPECTED_TEST_POINTS.length; i++)
         assertEquals(EXPECTED_RESULTS[i], 
               source.calculate(EXPECTED_TEST_POINTS[i]), 0.00001);
   }


   /**
    * Creates a CSVSource containing some data
    * 
    * @return A CSVSource
    * @throws IOException This shouldn't be able to happen....
    */
   private CSVSource createCSVSource() throws IOException
   {
      StringBuffer data = new StringBuffer();
      
      data.append("1, 1, 2\n");
      data.append("2, 1, 3\n");
      data.append("  #This is ignored\n");
      data.append("  \n");
      data.append("1, 2, 3\n");
      data.append("2, 2, 4\n");
      
      return new CSVSource(new StringReader(data.toString()));
   }
}
