package org.seatiger.pso.test;

import java.util.*;

import junit.framework.TestCase;

import org.seatiger.pso.Swarm;

public class SwarmTest extends TestCase
{

   public SwarmTest()
   {
      super("SwarmTest");
   }
	
   /*
    * Test method for 'org.seatiger.pso.Swarm.findParetoFronts(Collection<double[]>)'
    */
   public void testFindParetoFronts()
   {
      int numToTest = 100;
      int objectives = 2;
      
      ArrayList<double[]> testPoints = new ArrayList<double[]>(numToTest);
      for (int i = 0; i < numToTest; i++)
      {
         double[] temp = new double[objectives];
         for (int j = 0; j < objectives; j++)
            temp[j] = Math.random();
         
         testPoints.add(temp);
      }
      
      List<Collection<double[]>> paretoFronts = Swarm.findParetoFronts(testPoints);
      
      for (int i = 0; i < paretoFronts.size(); i++)
      {
         Iterator<double[]> outerIterator = paretoFronts.get(i).iterator();
         
         while (outerIterator.hasNext())
         {
            double[] outer = outerIterator.next();
            
            for (int j = 0; j < paretoFronts.size(); j++)
            {
               Iterator<double[]> innerIterator = paretoFronts.get(j).iterator();   
               
               while (innerIterator.hasNext())
               {
                  double[] inner = innerIterator.next();
                  int domination = Swarm.compareDomination(outer, inner);
                  
                  if (i < j)
                     assertTrue(domination <= 0);
                  else if (i > j)
                     assertTrue(domination >= 0);
                  else
                     assertTrue(domination == 0);
               }
            }
         }         
      }
   }

   /**
    * Test method for 'org.seatiger.pso.Swarm.compareDomination(double[], double[])'
    */
   public void testDomination()
   {
      double[] basePoint = {0, 0, 0};
      
//    This dominates the base point
      double[] topRight = {1, 1, 1}; 
      
//    These are in the same pareto-front as the base point
      double[] topLeft = {-1, 1, 0};
      double[] bottomRight = {1, -1, 0};
      
//    These are dominated by the base point
      double[] bottomLeft = {-1, -1, 0};
      double[] frontBottomLeft = {-1, -1, -1};
      
      
      assertTrue(Swarm.compareDomination(basePoint, topRight) > 0);
      assertTrue(Swarm.compareDomination(topRight, basePoint) < 0);
      
      assertEquals(0, Swarm.compareDomination(basePoint, topLeft));
      assertEquals(0, Swarm.compareDomination(basePoint, bottomRight));
      assertEquals(0, Swarm.compareDomination(basePoint, basePoint));
      
      assertTrue(Swarm.compareDomination(basePoint, bottomLeft) < 0);
      assertTrue(Swarm.compareDomination(basePoint, frontBottomLeft) < 0);
      
      assertTrue(Swarm.dominates(topRight, basePoint));
      assertTrue(!Swarm.dominates(basePoint, topRight));
      assertTrue(!Swarm.dominates(basePoint, topLeft));
      assertTrue(!Swarm.dominates(topLeft, basePoint));
      assertTrue(!Swarm.dominates(basePoint, basePoint));

   }


}
