package org.seatiger.pso.test;

import java.util.*;

import org.seatiger.pso.*;

public class TestTree
{
   static Range[] dataRange = new Range[] {new Range(0, 1), new Range(0, 1)};
   static BranchNode rootNode;
   static double[][] points;

   /**
    * @param args
    */
   public static void main(String[] args)
   {
      // Warm the JIT up
      treeInit(2000);
      treeFindNear();
      
      rootNode = null;
      points = null;
      
      Runtime runtime = Runtime.getRuntime();
      
      System.out.println(runtime.totalMemory()-runtime.freeMemory());
      System.gc();
      System.out.println(runtime.totalMemory()-runtime.freeMemory());
      testAccuracy();
   }
   
   public static void testTimings()
   {
      for (int i = 1; i <= 50000; i *= 2)
      {
         System.out.println("=== " + i + "===");
         long treeCulmInit = 0;
         long treeCulmFind = 0;
         long origCulmInit = 0;
         long origCulmFind = 0;


         for (int runs = 1; runs <= 50; runs++)
         {
            treeCulmInit += treeInit(i);
            origCulmInit += origInit(i);
            
            for (int j = 0; j < i; j++)
            {
               treeCulmFind += treeFindNear();
               origCulmFind += origFindNear();
            }
         }
         
         treeCulmInit /= 50;
         treeCulmFind /= 50;
         origCulmInit /= 50;
         origCulmFind /= 50;
         
         System.out.println("Tree: init: " + treeCulmInit + "ms find: " + treeCulmFind + "ms");
         System.out.println("Orig: init: " + origCulmInit + "ms find: " + origCulmFind + "ms");
      }
      
   }
   
   public static long treeInit(int points)
   {
      rootNode = new BranchNode(dataRange);
      long start = System.currentTimeMillis();

      for (int i = 0; i < points; i++)
         rootNode.add(new double[] {Math.random(), Math.random()});
      
      return System.currentTimeMillis()-start;
   }
   
   public static long treeFindNear()
   {
      long start = System.currentTimeMillis();
      Set<LeafNode> result = rootNode.getAllLeafNodesNear(new double[] {0.4, 0.4}, 0.01);
      return System.currentTimeMillis() - start;
   }

   public static long origInit(int numPoints)
   {
      long start = System.currentTimeMillis();
      points = new double[numPoints][];
      
      for (int i = 0; i < numPoints; i++)
         points[i] = new double[] {Math.random(), Math.random()};
      
      return System.currentTimeMillis()-start;
   }
   
   public static long origFindNear()
   {
      long start = System.currentTimeMillis();
      LinkedHashSet<double[]> result = new LinkedHashSet<double[]>();
      double[] location = {0.4, 0.4};
      
      for (int i = 0; i < points.length; i++)
      {
         if (VectorMath.length(VectorMath.subtract(points[i], location)) < 0.01)
            result.add(points[i]);
      }
      
      return System.currentTimeMillis()-start;
   }
   
   public static void testAccuracy()
   {
      final int NUM_POINTS = 800;
      final int RADIUS = 1;
      
      double[][] points = new double[NUM_POINTS][];
      double[][] distances = new double[points.length][points.length];
      int[] oldClosest = new int[points.length];
      LeafNode[] newClosest = new LeafNode[points.length];
      ArrayList<Set<Integer>> oldWithinRadius = new ArrayList<Set<Integer>>(); 
      ArrayList<Set<LeafNode>> newWithinRadius = new ArrayList<Set<LeafNode>>();
      
      for (int i = 0; i < points.length; i++)
         points[i] = new double[] {Math.random() * 10 - 5, Math.random() * 10 - 5};
      

      // ----- Old method -----
      
      long oldStart = System.currentTimeMillis();
      
      for (int i = 0; i < distances.length; i++)
      {
         for (int j = i + 1; j < distances.length; j++)
         {
            double distance = VectorMath.length(VectorMath.subtract(points[i], points[j]));
            distances[i][j] = distance;
            distances[j][i] = distance;
         }
      }
      
      long oldInsert = System.currentTimeMillis() - oldStart;
      
      Arrays.fill(oldClosest, -1);
      
      for (int i = 0; i < distances.length; i++)
      {
         oldWithinRadius.add(new HashSet<Integer>());
         
         for (int j = 0; j < distances.length; j++)
         {
            if (i == j)
               continue;
            
            if (oldClosest[i] == -1 || distances[i][j] < distances[i][oldClosest[i]])
               oldClosest[i] = j;
         }
      }
      
      long oldClosestTime = System.currentTimeMillis() - oldStart - oldInsert;

      for (int i = 0; i < distances.length; i++)
      {
         oldWithinRadius.add(new HashSet<Integer>());
         
         for (int j = 0; j < distances.length; j++)
         {
            if (distances[i][j] < RADIUS)
               oldWithinRadius.get(i).add(j);
         }
      }

      long oldRadiusTime =  System.currentTimeMillis() - oldStart - oldInsert - oldClosestTime;
      
      long oldTime = System.currentTimeMillis() - oldStart;
      
      System.out.println("Old method insert: " + oldInsert + "ms");
      System.out.println("Old method closest: " + oldClosestTime + "ms");
      System.out.println("Old method radius: " + oldRadiusTime + "ms");
      System.out.println("Old method total: " + oldTime + "ms");

      // --------- New method ----------
      
      long newStart = System.currentTimeMillis();
      
      BranchNode rootNode = 
         new BranchNode(new Range[] {new Range(-5, 5), new Range(-5, 5)});

      for (int i = 0; i < points.length; i++)
         rootNode.add(points[i]);
      
      long creationTime = System.currentTimeMillis() - newStart;
      
      for (int i = 0; i < points.length; i++)
      {
         NearestPointQuery query = new NearestPointQuery(points[i], rootNode);
         query.next();
         newClosest[i] = query.next();
      }
      
      long queryTime = System.currentTimeMillis() - newStart - creationTime;
      for (int i = 0; i < points.length; i++)         
         newWithinRadius.add(rootNode.getAllLeafNodesNear(points[i], RADIUS));
   
      long proximityTime = System.currentTimeMillis() - newStart - creationTime - queryTime;
   
      long newTime = System.currentTimeMillis() - newStart;
      System.out.println("New method insertion: " + creationTime + "ms");
      System.out.println("New method closest: " + queryTime + "ms");
      System.out.println("New method radius: " + proximityTime + "ms");
      System.out.println("New method total: " + newTime + "ms");
      
      boolean passed = true;
      System.out.print("Testing closest point: ");
      for (int i = 0; i < NUM_POINTS; i++)
      {
         double[] oldClosestPt = points[oldClosest[i]];
         double[] newClosestPt = newClosest[i].getPoint();
         
         double oldDist = VectorMath.length(VectorMath.subtract(oldClosestPt, points[i]));
         double newDist = VectorMath.length(VectorMath.subtract(newClosestPt, points[i]));
         
         if (!Arrays.equals(oldClosestPt, newClosestPt))
         {
            System.out.println("FAIL - old: " + oldDist + " new: " + newDist);
            
            passed = false;
         }
      }
      
      if (passed)
         System.out.println("PASSED");

      System.out.print("Testing particles within radius: ");
      
   }
}
