package peaks;

/**
 * <p>Title: PSOParameters.java </p>
 * <p>Description: Store all parameter associated with this PSO implementation </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: RMIT CS </p>
 * @author Xiaodong Li (xiaodong@cs.rmit.edu.au)
 * @version 1.0
 */

public class PSOParameters {
  // ---------------------------------------------------------------- Members

  /** The initial inertia weight **/
  private double initialWeight;

  /** The number of agent **/
  private int populationSize;

  /** The maximum velocity **/
  private double maximumVelocity;

  /** Left initial range **/
  private double leftInitialRange;

  /** Right initial range **/
  private double rightInitialRange;

  /** The maximum number of generations **/
  private int maximumIteration;

  /** The function name **/
  private String functionName;

  /** The number of dimensions in the specified function **/
  private int dimension;
  
  /** The delta value, ie the radius from the seed agent**/
  private double radius;

  /** Acceptance Threshold. If reached, then stop the run **/
  private double aThreshold;
  
  /** The max number of particles in a species **/
  private int speciesMax;

  /** method chosen to redistribute particles at each iteration **/
  private String method;
  
  // ---------------------------------------------------------- Constructors
  public PSOParameters(double initialWeight, int populationSize,
                       double maximumVelocity, double leftInitialRange,
                       double rightInitialRange, int maximumIteration,
                       String functionName, int dimension, double radius,
                       double aThreshold, int speciesMax, String method ) {

    this.initialWeight = initialWeight;
    this.populationSize = populationSize;
    this.maximumVelocity = maximumVelocity;
    this.leftInitialRange = leftInitialRange;
    this.rightInitialRange = rightInitialRange;
    this.maximumIteration = maximumIteration;
    this.functionName = functionName;
    this.dimension = dimension;
    this.radius = radius;
    this.aThreshold = aThreshold;
    this.speciesMax = speciesMax;
    this.method = method;
 }


  // ------------------------------------------------------------ Methods

  public double getInitialWeight(){
       return initialWeight;
  }

  public void setInitialWeight(double d) {
       initialWeight = d;
  }

   public int getPopulationSize() {
       return populationSize;
   }

   public void setPopulationSize(int i){
       populationSize = i;
   }

   public double getMaximumVelocity() {
       return maximumVelocity;
   }

   public void setMaximumVelocity(double d){
       maximumVelocity = d;
   }

   public double getLeftInitialRange() {
       return leftInitialRange;
   }

   public void setLeftInitialRange(double leftRange) {
       leftInitialRange = leftRange;
   }

   public double getRightInitialRange() {
       return rightInitialRange;
   }

   public void setRightInitialRange(double rightRange)  {
       rightInitialRange = rightRange;
   }

   public int getMaximumIteration() {
       return maximumIteration;
   }

   public void setMaximumIteration(int iter)  {
       maximumIteration = iter;
   }

   public String getFunctionName() {
       return functionName;
   }

   public void setFunctionName(String function) {
       functionName = function;
   }

   public int getFunctionDimension() {
       return dimension;
   }

   public void setFunctionDimension(int dim) {
       dimension = dim;
   }
 
   public double getRadiusValue() {
       return radius;
   }
 
   public void setRadiusValue(double r) {
       radius = r;
   }
 
   public double getAThreshold() {
       return aThreshold;
   }
 
   public void setAThreshold(double ath) {
       aThreshold = ath;
   }
 
   public int getSpeciesMax() {
       return speciesMax;
   }
 
   public void setSpeciesMax(int sm) {
       speciesMax = sm;
   }
     
   public String getMethod() {
       return method;
   }

   public void setMethod(String method) {
       this.method = method;
   }

}
